@@ -1,8 +1,8 @@
 from pyg_base._as_float import as_float
 from pyg_base._as_list import as_list, as_tuple, first, last, passthru, unique, is_rng
 from pyg_base._as_primitive import as_primitive
-from pyg_base._cache import cache, get_cache, cache_func
-from pyg_base._cfg import mkdir, cfg_read, cfg_write, CFG
+from pyg_base._cache import cache, cache_func
+from pyg_base._cfg import mkdir, get_cache, cfg_read, cfg_write, CFG
 from pyg_base._dates import iso, dt, uk2dt, us2dt, dt_bump, as_tz, tz_convert, today, ymd, TMIN, TMAX, DAY, futcodes, dt2str, is_bump, nth_weekday_of_month, mmm2m
 from pyg_base._decorators import kwargs_support, kwpartial, wrapper, try_value, try_back, try_nan, try_none, try_zero, try_false, try_true, try_list, timer
 from pyg_base._dict import Dict, items_to_tree, tree_items, tree_keys, tree_values, tree_setitem, tree_getitem, tree_get, tree_update, dict_invert
@@ -18,6 +18,7 @@ from pyg_base._loop import loop, loops, len0, pd2np, shape, loop_all, skip_if_da
 from pyg_base._named_dict import named_dict
 from pyg_base._pandas import df_index, df_columns, df_reindex, np_reindex, df_recolumn, df_concat, df_fillna, presync, df_sync, df_apply, \
     add_, sub_, mul_, div_, pow_, df_slice, df_unslice, nona, min_, max_, gt_, ge_, lt_, le_, df_count, df_sum, df_mean, df_std
+from pyg_base._pandas import ts_deal_with_issue, ts_gap, ts_degap, df_drop_index_duplicates
 from pyg_base._perdictable import join, perdictable
 from pyg_base._reducer import reducer, reducing
 from pyg_base._sort import sort, cmp, Cmp
